#!/bin/sh
set -e

TEMP=$(getopt -s 'sh' -o 'vg' -l 'verbose,generate' -n 'checkconfig.sh' -- "$@")
[ $? -ne 0 ] && exit 1

eval set -- "$TEMP"
unset TEMP

verbose=0
generate=0
while true; do
	case "$1" in
		'-v'|'--verbose')
			verbose=1
			shift
			continue
		;;
		'-g'|'--generate')
			generate=1
			shift
			continue
		;;
		'--')
			shift
			break
		;;
		*)
			echo 'getopt error...' >&2
			exit 1
		;;
	esac
done

if [ "$generate" -eq "0" ] && [ "$#" -lt 1 ]
then
	echo "Usage : $0 [-v] [-g] configfile" 2>/dev/null
	exit 1
fi

config=$1
err=0

if [ "$generate" -eq "0" ]
then
	conf=$(cat "$config")
fi

while read -r rule
do
	opt="$(echo "$rule" | cut -d ":" -f 1)"
	comment="$(echo "$rule" | cut -d ":" -f 2-)"
	if [ "$generate" -ne "0" ]
	then
		echo "$opt"
		continue
	fi
	ok=0
	if echo "$conf" | grep "$opt" >/dev/null
	then
		ok=1
	else
		if echo "$opt" | grep "^#" >/dev/null
		then
			if echo "$conf" | grep -v "$(echo "$opt" | sed -e 's/^# //' -e 's/ is not set.*$//')" >/dev/null
			then
				ok=1
			fi
		fi
	fi
	if [ "$ok" -eq 1 ]
	then
		if [ "$verbose" -ne 0 ]
		then
			printf "%-50s" "$opt"
			echo " [$(tput setaf 2)OK$(tput sgr0)]"
		fi
	else
		err=1
		printf "%-50s" "$opt"
		echo "[$(tput setaf 1)FAIL$(tput sgr0)] $comment"
	fi

done << __EOF__
# CONFIG_BLK_DEV_INITRD is not set:Initrd should be disabled
# CONFIG_MODULES is not set:Kernel should be monolitic (without module support)
# CONFIG_VIRTUALIZATION is not set:Do you want to run guest on guest ?
# CONFIG_VIRT_DRIVERS is not set:Do you want to run guest on guest ?
# CONFIG_VHOST_MENU is not set:Do you want to run guest on guest ?
CONFIG_HYPERVISOR_GUEST=y:Run as guest
CONFIG_PREEMPT_NONE_BUILD=y:Disable preemption for server
# CONFIG_PREEMPT_DYNAMIC is not set:Disable dynamic preemption
CONFIG_PREEMPT_NONE=y:Disable preemption for server
CONFIG_VIRTIO=y:Needed by VIRTIO_PCI and VIRTIO_BALOON
CONFIG_VIRTIO_BALLOON=y:Support for hot memory amount change for KVM guest
CONFIG_VIRTIO_BLK=y:The virtio block device driver
CONFIG_VIRTIO_CONSOLE=y:Support for virtio serial console
CONFIG_VIRTIO_INPUT=y:The virtio input (kbd, mice) driver
CONFIG_VIRTIO_IOMMU=y:Virtio IOMMU support
CONFIG_VIRTIO_MEM=y:The virtio memory driver
CONFIG_VIRTIO_MENU=y:The virtio drivers
CONFIG_VIRTIO_MMIO=y:Support for memory mapped virtio driver
CONFIG_VIRTIO_NET=y:The virtio network driver
CONFIG_VIRTIO_PCI=y:Support for virtio PCI devices
CONFIG_VIRTIO_VDPA=y:Virtio data path acceleration support
CONFIG_HW_RANDOM_VIRTIO=y:Virtio random number generator support
CONFIG_VP_VDPA=y:Bridges virtio PCI to vDPA
CONFIG_SCSI_VIRTIO=y:Virtual HBA driver for virtio
CONFIG_PCI_MSI=y:Allows driver to enable Message Signaled Interrupts
CONFIG_KVM_GUEST=y:Optimization for KVM guest
CONFIG_PARAVIRT=y:Optimization for linux guest
CONFIG_PARAVIRT_CLOCK=y:Optimization for linux guest
CONFIG_PARAVIRT_SPINLOCKS=y:Optimization for linux spinlocks on guest
CONFIG_MEMORY_HOTPLUG=y:Needed by VIRTIO_BALLOON
CONFIG_MEMORY_HOTREMOVE=y:Needed by VIRTIO_BALLOON
CONFIG_SERIAL_8250_CONSOLE=y:Needed by serial console
CONFIG_TMPFS=y:Needed for various pseudo-filesystems
CONFIG_DEVTMPFS=y:/dev pseudofilesystem
CONFIG_EXT4_FS=y:Needed by tests with debvm
CONFIG_SWAP=y:Swap support
CONFIG_STANDALONE=y:Select only drivers that don't need compile-time external firmware
CONFIG_PREVENT_FIRMWARE_BUILD=y:Disable drivers features which enable custom firmware building
# CONFIG_FW_CFG_SYSFS is not set:Disable qemu firmware controlling interface
# CONFIG_RPMSG_VIRTIO is not set:Not needed
CONFIG_CGROUPS=y:Cgroups support
CONFIG_UNIX=y:Unix socket support
CONFIG_PACKET=y:Packet socket support
CONFIG_CRYPTO_USER_API_RNG=y:Userspace interface for RNG
CONFIG_PAGE_TABLE_CHECK=y:Security hardening
CONFIG_SHUFFLE_PAGE_ALLOCATOR=y:Security hardening
CONFIG_SECURITY_DMESG_RESTRICT=y:Security hardening
CONFIG_SECURITY_PERF_EVENTS_RESTRICT=y:Security hardening
CONFIG_HARDENED_USERCOPY=y:Security hardening
CONFIG_FORTIFY_SOURCE=y:Security hardening
CONFIG_STATIC_USERMODEHELPER=y:Security hardening
CONFIG_STRICT_KERNEL_RWX=y:Security hardening
# CONFIG_DEBUG_FS is not set:Security hardening
CONFIG_STACKPROTECTOR=y:Security hardening
CONFIG_STACKPROTECTOR_STRONG=y:Security hardening
# CONFIG_DEVMEM is not set:Security hardening
CONFIG_SCHED_STACK_END_CHECK=y:Security hardening
CONFIG_HARDENED_USERCOPY=y:Security hardening
CONFIG_VMAP_STACK=y:Security hardening
CONFIG_FORTIFY_SOURCE=y:Security hardening
# CONFIG_PROC_KCORE is not set:Security hardening
CONFIG_SECURITY_DMESG_RESTRICT=y:Security hardening
CONFIG_RETPOLINE=y:Security hardening
CONFIG_LEGACY_VSYSCALL_NONE=y:Security hardening
# CONFIG_LEGACY_VSYSCALL_XONLY is not set:Security hardening
CONFIG_DEBUG_CREDENTIALS=y:Security hardening
CONFIG_DEBUG_NOTIFIERS=y:Security hardening
CONFIG_DEBUG_LIST=y:Security hardening
CONFIG_DEBUG_SG=y:Security hardening
CONFIG_BUG_ON_DATA_CORRUPTION=y:Security hardening
CONFIG_SLAB_FREELIST_RANDOM=y:Security hardening
CONFIG_SLUB=y:Security hardening
CONFIG_SLAB_FREELIST_HARDENED=y:Security hardening
# CONFIG_SLAB_MERGE_DEFAULT is not set:Security hardening
CONFIG_PAGE_POISONING=y:Security hardening
# CONFIG_COMPAT_BRK is not set:Security hardening
CONFIG_BUG=y:Security hardening
CONFIG_PANIC_ON_OOPS=y:Security hardening
CONFIG_PANIC_TIMEOUT=-1:Security hardening
CONFIG_SECURITY_YAMA=y:Security hardening
CONFIG_SECCOMP=y:Security hardening
CONFIG_SECCOMP_FILTER=y:Security hardening
CONFIG_SYN_COOKIES=y:Security hardening
# CONFIG_KEXEC is not set:Security hardening
# CONFIG_HIBERNATION is not set:Security hardening
# CONFIG_BINFMT_MISC is not set:Security hardening
# CONFIG_LEGACY_PTYS is not set:Security hardening
# CONFIG_ACPI_CUSTOM_METHOD is not set
# CONFIG_COMPAT_VDSO is not set
# CONFIG_LEGACY_VSYSCALL_EMULATE is not set
# CONFIG_X86_VSYSCALL_EMULATION is not set
# CONFIG_SECURITY_WRITABLE_HOOKS is not set
CONFIG_DEFAULT_MMAP_MIN_ADDR=65536:Security hardening
CONFIG_RANDOMIZE_BASE=y:Security hardening
CONFIG_RANDOMIZE_MEMORY=y:Security hardening
CONFIG_PAGE_TABLE_ISOLATION=y:Security hardening
# CONFIG_IA32_EMULATION is not set:Security hardening
# CONFIG_MODIFY_LDT_SYSCALL is not set:Security hardening
CONFIG_GCC_PLUGINS=y:Security hardening
CONFIG_GCC_PLUGIN_LATENT_ENTROPY=y:Security hardening
CONFIG_GCC_PLUGIN_STACKLEAK=y:Security hardening
CONFIG_RANDSTRUCT=y:Security hardening
CONFIG_RANDSTRUCT_FULL=y:Security hardening
__EOF__

if [ "$generate" -ne 0 ]
then
	echo "\
CONFIG_X86_PLATFORM_DEVICES=n
CONFIG_IOMMU_SUPPORT=y
# Needed by paravirt spinlocks
CONFIG_SMP=y
# Needed by virtio scsi
CONFIG_SCSI=y
CONFIG_SCSI_LOWLEVEL=y
CONFIG_BLK_DEV=y
# Needed by virtio net
CONFIG_NET=y
CONFIG_NETDEVICES=y
CONFIG_NET_CORE=y
CONFIG_INET=y
# PCI support
CONFIG_PCI=y
#Needed by serial console
CONFIG_SERIAL_8250=y
# Needed by virtio rng
CONFIG_HW_RANDOM=y
# Needed by virtio vdpa
CONFIG_VDPA=y
# Security hardening dependencies
CONFIG_SECURITY=y
CONFIG_SLUB_DEBUG_ON=y
CONFIG_ARCH_OPTIONAL_KERNEL_RWX=y
CONFIG_ARCH_HAS_STRICT_KERNEL_RWX=y
CONFIG_DEBUG_KERNEL=y
# Security hardening
CONFIG_RELOCATABLE=y
CONFIG_MODIFY_LDT_SYSCALL=n
CONFIG_EXPERT=y
"
fi


exit $err
